/* This testcase is part of GDB, the GNU debugger.

   Copyright 2011-2013 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */

namespace xxx
{
  namespace
  {
    static int func (void) { return 0; } // xxx::func
    class A
    {
    public:
      static int func (void) { return 0; } // xxx::A::func
    };
  }
}

int
test_function (void)
{
  return xxx::func () + xxx::A::func ();
}

int
main (void)
{
  int i, x;

  for (i = 0; i < 1000; ++i)
    x += test_function ();

  return x;
}
